
flush.tree <- function(tree,threshold) {

####################

starting.scipen=options("scipen")
starting.digits=options("digits")

starting.scipen=as.integer(starting.scipen)
starting.digits=as.integer(starting.digits)

options(scipen=999)
options(digits=8)

###

# extract some basic tree info

n.taxa=length(tree$tip.label)
n.nodes=tree$Nnode
root.node.number=n.taxa+1
last.node.number=root.node.number+n.nodes-1


###

# get starting tip ages

my.dist.matrix=dist.nodes(tree)
root.to.tips.dists=as.matrix(my.dist.matrix[1:n.taxa,root.node.number])

tallest.tip=max(root.to.tips.dists)
starting.tip.ages=(root.to.tips.dists-tallest.tip)*-1
#print(starting.tip.ages,digits=22)

###

# fix floating point issues with edge lengths

fp=256
orig.edge.lengths=tree$edge.length

edge.lengths=tree$edge.length
edge.lengths=as.numeric(edge.lengths)
edge.lengths=edge.lengths/(1/fp)
edge.lengths=round(edge.lengths,digits=0)
edge.lengths=edge.lengths*(1/fp)

new.edge.lengths=edge.lengths

zero.test=which(new.edge.lengths==0)
any.zeroes=length(zero.test)

if(any.zeroes>0) {
new.edge.lengths[zero.test]=1/fp
}

###

# compare new to old

comparison=cbind(orig.edge.lengths,new.edge.lengths)
#print(comparison,digits=22)
#plot(comparison,pch=20,cex=0.5,col="black")

###

# make tree with new branch lengths

new.tree=tree
new.tree$edge.length=new.edge.lengths

#dev.new(width=5,height=10); par(mar=c(0,0,0,0)); plot(tree,cex=0.75)
#dev.new(width=5,height=10); par(mar=c(0,0,0,0)); plot(new.tree,cex=0.75)

###

# get new tree tip ages

my.dist.matrix.new=dist.nodes(new.tree)
root.to.tips.dists.new=as.matrix(my.dist.matrix.new[1:n.taxa,root.node.number])

tallest.tip.new=max(root.to.tips.dists.new)
starting.tip.ages.new=(root.to.tips.dists.new-tallest.tip.new)*-1
#print(starting.tip.ages.new,digits=22)


###

# identify tips to adjust

tips.to.adjust=which(starting.tip.ages.new<threshold)
any.to.adjust=length(tips.to.adjust)

###

# identify terminal edges

parent.child.matrix=tree$edge
n.rows=dim(parent.child.matrix)[1]
edge.numbers=seq(from=1,to=n.rows,by=1)
parent.child.matrix=cbind(edge.numbers,parent.child.matrix)
colnames(parent.child.matrix)=c("edge.number","parent.node","child.node")

terminal.edge.numbers=which(parent.child.matrix[,3]<root.node.number)
terminal.parent.child.matrix=parent.child.matrix[terminal.edge.numbers,]

#print(terminal.parent.child.matrix)

###

# adjust lengths of terminal edges

if(any.to.adjust>0) {
terminal.edge.lengths=new.tree$edge.length[terminal.edge.numbers]
starting.tip.ages.new=as.numeric(starting.tip.ages.new)

new.terminal.edge.lengths=terminal.edge.lengths
new.terminal.edge.lengths[tips.to.adjust]=terminal.edge.lengths[tips.to.adjust]+starting.tip.ages.new[tips.to.adjust]

adjusted.new.edge.lengths=new.tree$edge.length
adjusted.new.edge.lengths[terminal.edge.numbers]=new.terminal.edge.lengths
#print(adjusted.new.edge.lengths,digits=22)

new.tree2=new.tree
new.tree2$edge.length=adjusted.new.edge.lengths
}

if(any.to.adjust==0) {
new.tree2=new.tree
}

###

# display change to flush

r2n=dist.nodes(new.tree2)
r2n=as.matrix(r2n[1:n.taxa,root.node.number])
max.tip=max(r2n)
new.tip.ages=(r2n-max.tip)*-1

taxon.names=tree$tip.label
tip.numbers=seq(from=1,to=n.taxa,by=1)

results.matrix=cbind(tip.numbers,starting.tip.ages,new.tip.ages)
colnames(results.matrix)=c("tip.number","original.tip.age","new.tip.age")
rownames(results.matrix)=taxon.names

msg=""
write.table(msg,quote=F,col.names=F,row.names=F)
flush.console()

print(results.matrix,digits=22)
flush.console()

msg=""
write.table(msg,quote=F,col.names=F,row.names=F)
flush.console()

####################

return(new.tree2) }
